@@ -149,7 +149,6 @@ sub filetype
 {
     my $self = shift( @_ );
     my $file = $self->{filepath};
-    # $self->message( 3, "Stating file '$file'" );
     CORE::stat( $file );
     if( !-e( _ ) )
     {
@@ -211,7 +210,6 @@ sub group
     # perlport: "getgrgid: (Win32, VMS, RISC OS) Not implemented."
     return( $self->gid ) if( $^O =~ /^(win32|vms|riscos)$/i );
     my $name = scalar( getgrgid( $data->[ FINFO_GID ] ) );
-    $self->message( 3, "Group name is defined? ", ( defined( $name ) ? "yes ($name)" : 'no' ) );
     return( $self->new_scalar( scalar( getgrgid( $data->[ FINFO_GID ] ) ) ) );
 }
 
